@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2005 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -91,45 +91,25 @@ extern void (*tramp) (); /* trampoline prototype */
 #include <stdio.h> /* declares fprintf() */
 
 #include <sys/types.h>
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
+#include <stdlib.h> /* declares abort(), malloc(), free() */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-/* Declare malloc(), free(). */
-#ifndef malloc
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETMALLOCTYPE malloc (MALLOC_SIZE_T size);
-#else
-extern RETMALLOCTYPE malloc();
-#endif
-#endif
-#ifndef free
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETFREETYPE free (RETMALLOCTYPE ptr);
-#else
-extern RETFREETYPE free();
-#endif
-#endif
-
-/* Declare abort(). */
-#if defined(__STDC__) || defined(__cplusplus)
-extern ABORT_VOLATILE RETABORTTYPE abort (void);
-#else
-extern ABORT_VOLATILE RETABORTTYPE abort();
-#endif
-
 /* Declare getpagesize(). */
 #ifdef HAVE_GETPAGESIZE
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETGETPAGESIZETYPE getpagesize (void);
+#ifdef __cplusplus
+extern "C" RETGETPAGESIZETYPE getpagesize (void);
 #else
-extern RETGETPAGESIZETYPE getpagesize ();
+extern RETGETPAGESIZETYPE getpagesize (void);
 #endif
 #else
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#else
+/* Not Unix, e.g. mingw32 */
+#define PAGESIZE 4096
+#endif
 #define getpagesize() PAGESIZE
 #endif
 
@@ -152,13 +132,6 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #else
 #include <sys/types.h>
 #include <sys/mman.h>
-#if !defined(__convex__)
-#if defined(__STDC__) || defined(__cplusplus)
-extern int mprotect (MPROTECT_CONST MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot);
-#else
-extern int mprotect ();
-#endif
-#endif
 #endif
 #endif
 #endif
@@ -170,11 +143,6 @@ extern int mprotect ();
 #if !defined(PROT_EXEC) && defined(PROT_EXECUTE) /* Irix 4.0.5 needs this */
 #define PROT_EXEC PROT_EXECUTE
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-extern RETMMAPTYPE mmap (MMAP_ADDR_T addr, MMAP_SIZE_T len, int prot, int flags, int fd, off_t off);
-#else
-extern RETMMAPTYPE mmap ();
-#endif
 #endif
 
 /* Declare open(). */
@@ -185,15 +153,6 @@ extern RETMMAPTYPE mmap ();
 #ifdef OPEN_NEEDS_SYS_FILE_H
 #include <sys/file.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-#ifdef OPEN_DOTS
-extern int open (OPEN_CONST char* path, int flags, ...);
-#else
-extern int open (OPEN_CONST char* path, int flags, mode_t mode);
-#endif
-#else
-extern int open ();
-#endif
 #endif
 
 /* Declare shmget(), shmat(), shmctl(). */
@@ -204,19 +163,6 @@ extern int open ();
 #ifdef HAVE_SYS_SYSMACROS_H
 #include <sys/sysmacros.h>
 #endif
-#if defined(__STDC__) || defined(__cplusplus)
-extern int shmget (key_t key, SHMGET_SIZE_T size, int shmflg);
-extern RETSHMATTYPE shmat (int shmid, SHMAT_CONST RETSHMATTYPE shmaddr, int shmflg);
-#ifdef SHMCTL_DOTS
-extern int shmctl (int shmid, int cmd, ...);
-#else
-extern int shmctl (int shmid, int cmd, struct shmid_ds * buf);
-#endif
-#else
-extern int shmget ();
-extern RETSHMATTYPE shmat ();
-extern int shmctl ();
-#endif
 #endif
 
 /* Support for instruction cache flush. */
@@ -242,7 +188,7 @@ extern int shmctl ();
 #include <mips/cachectl.h>
 #else
 #ifdef linux
-#include <linux/cachectl.h>
+#include <asm/cachectl.h>
 #else
 #ifdef HAVE_SYS_CACHECTL_H
 #include <sys/cachectl.h>
@@ -351,6 +297,14 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 40
 #define TRAMP_ALIGN 16
 #endif
+#ifdef __x86_64__
+#define TRAMP_LENGTH 32
+#define TRAMP_ALIGN 16
+#endif
+#ifdef __s390__
+#define TRAMP_LENGTH 36
+#define TRAMP_ALIGN 2
+#endif
 
 #ifndef TRAMP_BIAS
 #define TRAMP_BIAS 0
@@ -364,14 +318,7 @@ extern void __TR_clear_cache();
 static char* freelist = NULL;
 #endif
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function alloc_trampoline (__TR_function address, void* variable, void* data)
-#else
-__TR_function alloc_trampoline (address, variable, data)
-  __TR_function address;
-  void* variable;
-  void* data;
-#endif
 {
   char* function;
 
@@ -881,13 +828,16 @@ __TR_function alloc_trampoline (address, variable, data)
    *    .long   <data>
    *    .long   <address>
    */
-  *(long *) (function + 0) = ((long *) ((char*)&tramp-2))[0];
-  *(long *) (function + 4) = (long) (function + 8);
-  *(long *) (function + 8) = (long) variable;
-  *(long *) (function +12) = (long) data;
-  *(long *) (function +16) = (long) address;
+  { /* work around a bug in gcc 3.* */
+    void* tramp_address = &tramp;
+    *(long *) (function + 0) = ((long *) ((char*)tramp_address-2))[0];
+    *(long *) (function + 4) = (long) (function + 8);
+    *(long *) (function + 8) = (long) variable;
+    *(long *) (function +12) = (long) data;
+    *(long *) (function +16) = (long) address;
+  }
 #define is_tramp(function)  \
-  ((long *) function)[0] == ((long *) ((char*)&tramp-2))[0]
+  ((long *) function)[0] == ((long *) ((char*)tramp_address-2))[0]
 #define tramp_address(function)  \
   ((long *) function)[4]
 #define tramp_variable(function)  \
@@ -1101,6 +1051,93 @@ __TR_function alloc_trampoline (address, variable, data)
 #define tramp_data(function)  \
   ((long *) function)[4]
 #endif
+#ifdef __x86_64__
+  /* function:
+   *    movabsq $<data>,%rax		48 B8 <data>
+   *    movabsq %rax, <variable>	48 A3 <variable>
+   *    movabsq $<address>,%rax		48 B8 <address>
+   *    jmp *%rax			FF E0
+   */
+  *(short *) (function + 0) = 0xB848;
+  *(short *) (function + 2) = (unsigned long) data & 0xffff;
+  *(int *)   (function + 4) = ((unsigned long) data >> 16) & 0xffffffff;
+  *(short *) (function + 8) = ((unsigned long) data >> 48) & 0xffff;
+  *(short *) (function +10) = 0xA348;
+  *(int *)   (function +12) = (unsigned long) variable & 0xffffffff;
+  *(int *)   (function +16) = ((unsigned long) variable >> 32) & 0xffffffff;
+  *(short *) (function +20) = 0xB848;
+  *(short *) (function +22) = (unsigned long) address & 0xffff;
+  *(int *)   (function +24) = ((unsigned long) address >> 16) & 0xffffffff;
+  *(short *) (function +28) = ((unsigned long) address >> 48) & 0xffff;
+  *(short *) (function +30) = 0xE0FF;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0xB848 && \
+  *(unsigned short *) (function +10) == 0xA348 && \
+  *(unsigned short *) (function +20) == 0xB848 && \
+  *(unsigned short *) (function +30) == 0xE0FF
+#define hilo(hiword,loword)  \
+  (((unsigned long) (hiword) << 32) | (unsigned long) (loword))
+#define himidlo(hishort,midword,loshort)  \
+  (((unsigned long) (hishort) << 48) | (unsigned long) (midword) << 16 \
+   | (unsigned long) (loshort))
+#define tramp_address(function)  \
+  himidlo(*(unsigned short *) (function +28), \
+          *(unsigned int *)   (function +24), \
+          *(unsigned short *) (function +22))
+#define tramp_variable(function)  \
+  hilo(*(unsigned int *) (function +16), *(unsigned int *) (function +12))
+#define tramp_data(function)  \
+  himidlo(*(unsigned short *) (function + 8), \
+          *(unsigned int *)   (function + 4), \
+          *(unsigned short *) (function + 2))
+#endif
+#ifdef __s390__
+  /* function:
+
+        lr      %r0,%r13
+        bras    %r13,.LTN0_0
+.LT0_0:
+.LC0:
+        .long   0x73554711
+.LC1:
+        .long   0x12345678
+.LC2:
+        .long   0xbabebec0
+.LTN0_0:
+        l       %r1,.LC0-.LT0_0(%r13)
+        mvc     0(4,%r1),.LC1-.LT0_0(%r13)
+        l       %r1,.LC2-.LT1_0(%r13)
+        lr      %r13,%r0
+        br      %r1
+  */
+  /* What about big endian / little endian ?? */
+  *(unsigned short *) (function + 0) = 0x180D;
+  *(unsigned int *)   (function + 2) = 0xA7D50008;
+  *(unsigned int *)   (function + 6) = (unsigned int) variable;
+  *(unsigned int *)   (function +10) = (unsigned int) data;
+  *(unsigned int *)   (function +14) = (unsigned int) address;
+  *(unsigned int *)   (function +18) = 0x5810D000;
+  *(unsigned int *)   (function +22) = 0xD2031000;
+  *(unsigned short *) (function +26) = 0xD004;
+  *(unsigned int *)   (function +28) = 0x5810D008;
+  *(unsigned short *) (function +32) = 0x18D0;
+  *(unsigned short *) (function +34) = 0x07f1;
+#define is_tramp(function)  \
+  *(short *)          (function + 0) == 0x180D && \
+  *(int *)            (function + 2) == 0xA7D50008 && \
+  *(int *)            (function +18) == 0x5810D000 && \
+  *(int *)            (function +22) == 0xD2031000 && \
+  *(short *)          (function +26) == 0xD004 && \
+  *(int *)            (function +28) == 0x5810D008 && \
+  *(short *)          (function +32) == 0x18D0 && \
+  *(short *)          (function +34) == 0x07f1
+#define tramp_address(function)  \
+  *(unsigned int *) (function +14)
+#define tramp_variable(function)  \
+  *(unsigned int *) (function +6)
+#define tramp_data(function)  \
+  *(unsigned int *) (function +10)
+#endif
 
   /* 3. Set memory protection to "executable" */
 
@@ -1122,7 +1159,7 @@ __TR_function alloc_trampoline (address, variable, data)
 #if defined(HAVE_SYS_M88KBCS_H)
     if (memctl(start_addr, len, MCT_TEXT) == -1)
 #else
-    if (mprotect((MMAP_ADDR_T)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
+    if (mprotect((void*)start_addr, len, PROT_READ|PROT_WRITE|PROT_EXEC) < 0)
 #endif
 #endif
 #endif
@@ -1233,12 +1270,7 @@ __TR_function alloc_trampoline (address, variable, data)
   return (__TR_function) (function + TRAMP_BIAS);
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void free_trampoline (__TR_function function)
-#else
-void free_trampoline (function)
-  __TR_function function;
-#endif
 {
 #if TRAMP_BIAS
   function = (__TR_function)((char*)function - TRAMP_BIAS);
@@ -1251,15 +1283,11 @@ void free_trampoline (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 int is_trampoline (void* function)
-#else
-int is_trampoline (function)
-  void* function;
-#endif
 {
 #ifdef is_tramp
 #ifdef __hppanew__
+  void* tramp_address = &tramp;
   if (!(((long)function & 3) == (TRAMP_BIAS & 3))) return 0;
 #endif
   return ((is_tramp(((char*)function - TRAMP_BIAS))) ? 1 : 0);
@@ -1268,12 +1296,7 @@ int is_trampoline (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function trampoline_address (void* function)
-#else
-__TR_function trampoline_address (function)
-  void* function;
-#endif
 {
 #ifdef tramp_address
   return (__TR_function)(tramp_address(((char*)function - TRAMP_BIAS)));
@@ -1282,12 +1305,7 @@ __TR_function trampoline_address (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_variable (void* function)
-#else
-void* trampoline_variable (function)
-  void* function;
-#endif
 {
 #ifdef tramp_variable
   return (void*)(tramp_variable(((char*)function - TRAMP_BIAS)));
@@ -1296,12 +1314,7 @@ void* trampoline_variable (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_data (void* function)
-#else
-void* trampoline_data (function)
-  void* function;
-#endif
 {
 #ifdef tramp_data
   return (void*)(tramp_data(((char*)function - TRAMP_BIAS)));
