@@ -4,7 +4,7 @@
  * rewritten by Sam Steingold 2003-08-06
  */
 
-#include <clisp.h>
+#include "clisp.h"
 #include <sys/types.h>          /* regex.h needs this */
 #include <stdlib.h>             /* declare malloc(), free() */
 #include <stdio.h>              /* BUFSIZ */
@@ -19,13 +19,11 @@
 
 DEFMODULE(regexp,"REGEXP")
 
+DEFFLAGSET(regexp_compile_flags, REG_EXTENDED REG_ICASE REG_NEWLINE REG_NOSUB)
 DEFUN(REGEXP::REGEXP-COMPILE, pattern &key EXTENDED IGNORE-CASE NEWLINE NOSUB)
 { /* compile the pattern into a regular expression */
-  object pattern = check_string(STACK_4);
-  int cflags = (missingp(STACK_0) ? 0 : REG_NOSUB) |
-    (missingp(STACK_1) ? 0 : REG_NEWLINE) |
-    (missingp(STACK_2) ? 0 : REG_ICASE) |
-    (missingp(STACK_3) ? 0 : REG_EXTENDED);
+  int cflags = regexp_compile_flags();
+  object pattern = check_string(popSTACK());
   int status;
   regex_t* re;
  restart_regcomp:
@@ -42,46 +40,54 @@ DEFUN(REGEXP::REGEXP-COMPILE, pattern &key EXTENDED IGNORE-CASE NEWLINE NOSUB)
     free(re);
     end_system_call();
     pushSTACK(NIL); /* no PLACE */
-    pushSTACK(asciz_to_string(buf,GLO(misc_encoding)));
-    pushSTACK(pattern); pushSTACK(TheSubr(subr_self)->name);
-    check_value(error,"~ (~): ~");
+    pushSTACK(NIL); pushSTACK(pattern);
+    STACK_1 = asciz_to_string(buf,GLO(misc_encoding));
+    pushSTACK(TheSubr(subr_self)->name);
+    check_value(error,"~S (~S): ~S");
     pattern = value1;
     goto restart_regcomp;
   }
-  STACK_2 = STACK_1 = allocate_fpointer((FOREIGN)re);
-  STACK_0 = ``REGEXP::REGEXP-FREE``; funcall(L(finalize),2);
-  VALUES1(STACK_0);
-  skipSTACK(3);
+  pushSTACK(allocate_fpointer((FOREIGN)re));
+  pushSTACK(STACK_0);pushSTACK(``REGEXP::REGEXP-FREE``);funcall(L(finalize),2);
+  VALUES1(popSTACK());          /* foreign pointer */
 }
 
 DEFUN(REGEXP::REGEXP-FREE, compiled)
 { /* release the contents and the data of the compiled pattern */
-  if (fpointerp(STACK_0) && fp_validp(TheFpointer(STACK_0))) {
-    regex_t *re = (regex_t*)TheFpointer(STACK_0)->fp_pointer;
-    regfree(re);
-    free(re);
-    VALUES1(T);
-  } else
-    VALUES1(NIL);
-  skipSTACK(1);
+  object fp = popSTACK();
+  if (fpointerp(fp) && fp_validp(TheFpointer(fp))) {
+    regex_t *re = (regex_t*)TheFpointer(fp)->fp_pointer;
+    if (re) {
+      regfree(re); free(re);
+      TheFpointer(fp)->fp_pointer = NULL;
+      mark_fp_invalid(TheFpointer(fp));
+      VALUES1(T);
+    } else VALUES1(NIL);
+  } else VALUES1(NIL);
 }
 
+DEFFLAGSET(regexp_exec_flags, REG_NOTBOL REG_NOTEOL)
 DEFUN(REGEXP::REGEXP-EXEC, pattern string &key BOOLEAN START END NOTBOL NOTEOL)
 { /* match the compiled pattern against the string */
-  object string = (STACK_5 = check_string(STACK_5));
+  int eflags = regexp_exec_flags();
+  object string = (STACK_3 = check_string(STACK_3));
   unsigned int length = vector_length(string);
-  unsigned int start = missingp(STACK_3) ? 0
-    : posfixnum_to_L(STACK_3 = check_posfixnum(STACK_3));
-  unsigned int end = missingp(STACK_2) ? length
-    : posfixnum_to_L(STACK_2 = check_posfixnum(STACK_2));
-  int eflags = ((missingp(STACK_0) ? 0 : REG_NOTEOL) |
-                (missingp(STACK_1) ? 0 : REG_NOTBOL));
+  unsigned int start = check_uint_defaulted(STACK_1,0);
+  unsigned int end = check_uint_defaulted(STACK_0,length);
   int status;
-  bool bool_p = !missingp(STACK_4);
+  bool bool_p = !missingp(STACK_2);
   regex_t *re;
   regmatch_t *ret;
-  skipSTACK(5);                 /* drop all options */
-  STACK_1 = check_fpointer(STACK_1,true);
+  skipSTACK(3);                 /* drop all options */
+  for (;;) {
+    STACK_1 = check_fpointer(STACK_1,true);
+    re = (regex_t*)TheFpointer(STACK_1)->fp_pointer;
+    if (re != NULL) break;
+    pushSTACK(NIL);             /* no PLACE */
+    pushSTACK(STACK_(1+1)); pushSTACK(TheSubr(subr_self)->name);
+    check_value(error,GETTEXT("~S: NULL pattern ~S"));
+    STACK_1 = value1;
+  }
   string = STACK_0;
   if (end != length || start != 0) {
     pushSTACK(sfixnum((int)(end-start)));
@@ -91,7 +97,6 @@ DEFUN(REGEXP::REGEXP-EXEC, pattern string &key BOOLEAN START END NOTBOL NOTEOL)
     funcall(L(make_array),7);
     string = value1;
   }
-  re = (regex_t*)TheFpointer(STACK_1)->fp_pointer;
   begin_system_call();
   ret = (regmatch_t*)alloca((re->re_nsub+1)*sizeof(regmatch_t));
   end_system_call();
